package cc.fascinated.bat.event;

import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import lombok.NonNull;
import net.dv8tion.jda.api.events.guild.member.GuildMemberJoinEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRemoveEvent;
import net.dv8tion.jda.api.events.interaction.component.StringSelectInteractionEvent;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;

/**
 * @author Fascinated (fascinated7)
 */
public interface EventListener {
    /**
     * Called when a ScoreSaber score is received
     *
     * @param score the score that was set
     * @param leaderboard the leaderboard that the score was set on
     * @param player the player that set the score
     */
    default void onScoresaberScoreReceived(@NonNull ScoreSaberPlayerScoreToken score, @NonNull ScoreSaberLeaderboardToken leaderboard,
                                           @NonNull ScoreSaberScoreToken.LeaderboardPlayerInfo player) {}

    /**
     * Called when a user joins a guild
     *
     * @param guild the guild the user joined
     * @param user the user that joined the guild
     */
    default void onGuildMemberJoin(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberJoinEvent event) {}

    /**
     * Called when a user leaves a guild
     *
     * @param guild the guild the user left
     * @param user the user that left the guild
     */
    default void onGuildMemberLeave(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberRemoveEvent event) {}

    /**
     * Called when a user types a message
     *
     * @param guild the guild that the message was sent in
     * @param user the user that sent the message
     */
    default void onGuildMessageReceive(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull MessageReceivedEvent event) {}

    /**
     * Called when a user selects a string
     *
     * @param guild the guild that the string was selected in
     * @param user the user that selected the string
     */
    default void onStringSelectInteraction(BatGuild guild, @NonNull BatUser user, @NonNull StringSelectInteractionEvent event) {}
}
