package cc.fascinated.bat.features.autorole.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.RoleUtils;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("autoroles:add.sub")
public class AddSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public AddSubCommand(@NonNull GuildService guildService) {
        super("add", "Adds a role to the auto roles list");
        super.addOption(OptionType.ROLE, "role", "The role to add", true);
        this.guildService = guildService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        if (profile.getRoleSlotsInUse() >= profile.getMaxRoles()) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You can only have a maximum of %d roles set for the auto role feature"
                    .formatted(profile.getMaxRoles()))
                    .build()).queue();
            return;
        }

        OptionMapping option = interaction.getOption("role");
        if (option == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Please provide a role to add")
                    .build()).queue();
            return;
        }

        Role role = option.getAsRole();
        if (profile.hasRole(role.getId())) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The role %s is already in the auto roles list".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        if (!RoleUtils.hasPermissionToGiveRole(guild, guild.getDiscordGuild().getSelfMember(), role)) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("I do not have permission to give the role %s".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        profile.addRole(role.getId());
        guildService.saveGuild(guild);
        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully added the %s role to the auto roles list".formatted(role.getAsMention()))
                .build()).queue();
    }
}
