package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.common.Colors;
import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.exception.RateLimitException;
import cc.fascinated.bat.features.scoresaber.profile.UserScoreSaberProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ScoreSaberCommand extends BatCommand {
    private final ScoreSaberService scoreSaberService;

    @Autowired
    public ScoreSaberCommand(@NonNull ScoreSaberService scoreSaberService, @NonNull ApplicationContext context) {
        super("scoresaber", "General ScoreSaber commands");
        this.scoreSaberService = scoreSaberService;

        super.addSubCommand("link", context.getBean(LinkSubCommand.class));
        super.addSubCommand("user", context.getBean(UserSubCommand.class));
        super.addSubCommand("me", context.getBean(MeSubCommand.class));
        super.addSubCommand("reset", context.getBean(ResetSubCommand.class));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        sendProfileEmbed(true, user, scoreSaberService, interaction);
    }

    /**
     * Builds the profile embed for the ScoreSaber profile
     *
     * @param user              The user to build the profile embed for
     * @param scoreSaberService The ScoreSaber service
     * @param interaction       The interaction
     */
    public static void sendProfileEmbed(boolean isSelf, BatUser user, ScoreSaberService scoreSaberService, SlashCommandInteraction interaction) {
        UserScoreSaberProfile profile = user.getProfile(UserScoreSaberProfile.class);
        if (profile.getSteamId() == null) {
            if (!isSelf) {
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("%s does not have a linked ScoreSaber account".formatted(user.getDiscordUser().getAsMention()))
                        .build()).queue();
            }
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You do not have a linked ScoreSaber account")
                    .build()).queue();
            return;
        }

        try {
            long before = System.currentTimeMillis();
            ScoreSaberAccountToken account = scoreSaberService.getAccount(profile.getSteamId());
            if (account == null) {
                if (!isSelf) {
                    interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("%s has an invalid ScoreSaber account linked, please ask them to re-link their account"
                                    .formatted(user.getDiscordUser().getAsMention()))
                            .build()).queue();
                }
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("You have an invalid ScoreSaber account linked, please re-link your account")
                        .build()).queue();
                return;
            }

            long fetchTime = System.currentTimeMillis() - before;
            interaction.replyEmbeds(new EmbedBuilder()
                    .setAuthor(account.getName() + "'s Profile", "https://scoresaber.com/u/%s".formatted(account.getId()),
                            "https://cdn.scoresaber.com/avatars/%s.jpg".formatted(account.getId()))
                    .addField("Name", account.getName(), true)
                    .addField("Country", account.getCountry(), true)
                    .addField("Rank", "#" + NumberUtils.formatNumberCommas(account.getRank()), true)
                    .addField("Country Rank", "#" + NumberUtils.formatNumberCommas(account.getCountryRank()), true)
                    .addField("PP", NumberUtils.formatNumberCommas(account.getPp()), true)
                    .addField("Joined", "<t:%s>".formatted(DateUtils.getDateFromString(account.getFirstSeen()).toInstant().toEpochMilli() / 1000), true)
                    .setTimestamp(LocalDateTime.now())
                    .setFooter(fetchTime > 3 ? "Fetched in %sms".formatted(fetchTime) : "Cached", null)
                    .setColor(Colors.DEFAULT)
                    .build()).queue();
        } catch (RateLimitException ex) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The ScoreSaber API is currently rate our limiting requests, please try again later")
                    .build()).queue();
        }
    }
}
