package cc.fascinated.bat.service;

import cc.fascinated.bat.common.TimerUtils;
import lombok.Getter;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.requests.GatewayIntent;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.EnumSet;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Service @Getter
public class DiscordService {
    /**
     * The JDA instance
     */
    public static JDA JDA;

    private final List<String> messages = List.of(
            "over {guilds} guilds",
            "over {users} users",
            "over ScoreSaber scores",
            "your messages",
            "/help for help"
    );

    @Autowired
    public DiscordService(
            @Value("${discord.token}") String token
    ) throws Exception {
        JDA = JDABuilder.create(token, EnumSet.of(
                        GatewayIntent.GUILD_MESSAGES,
                        GatewayIntent.MESSAGE_CONTENT,
                        GatewayIntent.GUILD_MEMBERS
                ))
                .disableCache(
                        CacheFlag.ACTIVITY,
                        CacheFlag.VOICE_STATE,
                        CacheFlag.EMOJI,
                        CacheFlag.STICKER,
                        CacheFlag.CLIENT_STATUS,
                        CacheFlag.ONLINE_STATUS,
                        CacheFlag.SCHEDULED_EVENTS
                ).build()
                .awaitReady();
        TimerUtils.scheduleRepeating(this::updateActivity, 0, 1000 * 60 * 2);
    }

    /**
     * Updates the activity of the bot
     */
    public void updateActivity() {
        int guildCount = JDA.getGuilds().size();
        JDA.getPresence().setActivity(Activity.watching(messages.get(guildCount % messages.size())
                .replace("{guilds}", String.valueOf(guildCount))
                .replace("{users}", String.valueOf(JDA.getUsers().size()))
        ));
    }
}
